==> yawp-1.21.4-fabric-0.5.2-beta1-sources.jar: de/z0rdak/yawp/handler/HandlerUtil.java <==
package de.z0rdak.yawp.handler;

import de.z0rdak.yawp.api.events.region.FlagCheckEvent;
import de.z0rdak.yawp.api.events.region.FlagCheckResult;
import de.z0rdak.yawp.api.permission.Permissions;
import de.z0rdak.yawp.core.flag.*;
import de.z0rdak.yawp.core.region.IMarkableRegion;
import de.z0rdak.yawp.core.region.IProtectedRegion;
import de.z0rdak.yawp.data.region.RegionDataManager;
import de.z0rdak.yawp.platform.Services;
import org.jetbrains.annotations.Nullable;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

import java.util.*;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import net.minecraft.class_1297;
import net.minecraft.class_1307;
import net.minecraft.class_1429;
import net.minecraft.class_1480;
import net.minecraft.class_1506;
import net.minecraft.class_1507;
import net.minecraft.class_1510;
import net.minecraft.class_1569;
import net.minecraft.class_1606;
import net.minecraft.class_1621;
import net.minecraft.class_1657;
import net.minecraft.class_1915;
import net.minecraft.class_1936;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_5321;

import static de.z0rdak.yawp.core.flag.RegionFlag.MOB_GRIEFING;

public final class HandlerUtil {

    private HandlerUtil() {
    }

    public static class_5321<class_1937> getDimKey(class_1297 entity) {
        return entity.method_37908().method_27983();
    }

    public static class_5321<class_1937> getDimKey(class_1937 world) {
        return world.method_27983();
    }

    public static boolean isAnimal(class_1297 entity) {
        boolean isMonsterHorse = entity instanceof class_1507 || entity instanceof class_1506;
        // Note: because monster horses are still horses and horses are animals? what the frick mojang
        return entity instanceof class_1429 && !isMonsterHorse 
                || entity instanceof class_1480;
    }

    public static boolean isServerSide(class_1297 entity) {
        return isServerSide(entity.method_37908());
    }

    public static boolean isServerSide(class_1936 world) {
        return !world.method_8608();
    }
    public static boolean isServerSide(class_1937 world) {
        return !world.field_9236;
    }

    public static boolean isVillager(class_1297 entity) {
        return entity instanceof class_1915;
    }

    public static boolean isPlayer(class_1297 entity) {
        return entity instanceof class_1657;
    }

    public static boolean isMonster(class_1297 entity) {
        return entity instanceof class_1569
                || entity instanceof class_1621
                || entity instanceof class_1307
                || entity instanceof class_1510
                || entity instanceof class_1606 
                || entity instanceof class_1507 || entity instanceof class_1506;
    }

    public static boolean notServerSideOrPlayerNull(class_1297 entity) {
        return entity == null || !isServerSide(entity);
    }

    /**
     * Processes the given flag check event and executes the given consumers if the flag is allowed or denied. <br>
     * The flag check event is evaluated and posted to the event bus. <br>
     * If the flag is allowed, the onAllow consumer is executed, otherwise onDeny. <br>
     *
     * @param checkEvent the flag check event to process
     * @param onAllow    the consumer to execute if the flag is allowed
     * @param onDeny     the consumer to execute if the flag is denied
     * @return the flag state of the result
     */
    public static FlagState processCheck(FlagCheckEvent checkEvent, @Nullable Consumer<FlagCheckResult> onAllow, @Nullable Consumer<FlagCheckResult> onDeny) {
        FlagCheckResult result = evaluate(checkEvent);
        result = Services.EVENT.post(result);
        if (result.getFlagState() == FlagState.ALLOWED
                || result.getFlagState() == FlagState.UNDEFINED
                || result.getFlagState() == FlagState.DISABLED) {
            if (onAllow != null)
                onAllow.accept(result);
        }
        if (result.getFlagState() == FlagState.DENIED) {
            if (onDeny != null)
                onDeny.accept(result);
        }
        return result.getFlagState();
    }

    public static FlagState processCheck(FlagCheckEvent checkEvent, @Nullable Consumer<FlagCheckResult> onDeny) {
        return processCheck(checkEvent, null, onDeny);
    }

    public static FlagState processCheck(FlagCheckEvent checkEvent) {
        return processCheck(checkEvent, null, null);
    }

    public static void checkMobGrief(class_1297 entity, CallbackInfo ci) {
        checkMobGrief(entity.method_37908(), entity.method_23312(), ci);
    }

    public static void checkMobGrief(class_1297 entity, CallbackInfoReturnable<Boolean> cir) {
        checkMobGrief(entity.method_37908(), entity.method_23312(), cir);
    }

    public static void checkMobGrief(class_1937 world, class_2338 pos, CallbackInfo ci) {
        if (isServerSide(world)) {
            FlagCheckEvent checkEvent = new FlagCheckEvent(pos, MOB_GRIEFING, world.method_27983());
            if (Services.EVENT.post(checkEvent))
                return;
            processCheck(checkEvent, deny -> ci.cancel());
        }
    }

    public static void checkMobGrief(class_1937 world, class_2338 pos, CallbackInfoReturnable<Boolean> cir) {
        if (isServerSide(world)) {
            FlagCheckEvent checkEvent = new FlagCheckEvent(pos, MOB_GRIEFING, world.method_27983());
            if (Services.EVENT.post(checkEvent))
                return;
            processCheck(checkEvent, deny -> cir.setReturnValue(false));
        }
    }

    public static void syncPlayerInventory(class_1937 world, class_1657 player) {
        // TODO:
    }

    public static void updateBlockState(class_1937 world, class_2338 pos) {
        world.method_61271(pos, world.method_8320(pos).method_26204());
    }

    /**
     * Evaluates the given flag check event and returns the result. <br>
     * The responsible region is determined and the flag state is checked against the player's permissions. <br>
     *
     * @param checkEvent the flag check event to evaluate
     * @return the result of the flag check event
     */
    public static FlagCheckResult evaluate(FlagCheckEvent checkEvent) {
        RegionFlag regionFlag = checkEvent.getRegionFlag();
        IProtectedRegion targetRegion = getResponsible(checkEvent.getTarget(), checkEvent.getDimension());
        if (targetRegion == null) {
            return FlagCheckResult.Undefined(checkEvent);
        }
        FlagMessage.FlagCorrelation responsibleFlag = getResponsibleFlag(targetRegion, regionFlag, null);
        FlagState playerRelatedState = getFlagState(responsibleFlag.getRegion(), regionFlag, checkEvent.getPlayer());
        return new FlagCheckResult(checkEvent, playerRelatedState, responsibleFlag.getRegion(), responsibleFlag.getFlag());
    }

    /**
     * Gets the responsible region for the given position and dimension. <br>
     * The responsible region is the region with the highest priority among all involved regions at the given location and dimension. <br>
     *
     * @param pos the position to get the responsible region for
     * @param dim the dimension to get the responsible region for
     * @return the responsible region for the given position and dimension
     */
    @Nullable
    private static IProtectedRegion getResponsible(class_2338 pos, class_5321<class_1937> dim) {
        IMarkableRegion region = getInvolvedRegionFor(pos, dim);
        if (region == null) {
            IProtectedRegion dimRegion = RegionDataManager.get().cacheFor(dim).getDimensionalRegion();
            if (dimRegion.isActive()) {
                return dimRegion;
            } else {
                return RegionDataManager.get().getGlobalRegion().isActive()
                        ? RegionDataManager.get().getGlobalRegion()
                        : null;
            }
        }
        return region;
    }

    /**
     * Gets all active regions which contain the provided position in the given dimension. <br>
     *
     * @param position the position to check for involved regions
     * @param dim      the dimension to check for involved regions
     * @return all active regions which contain the given location and dimension
     */
    private static List<IMarkableRegion> getInvolvedRegionsFor(class_2338 position, class_5321<class_1937> dim) {
        return RegionDataManager.get().getRegionsFor(dim).stream()
                .filter(IMarkableRegion::isActive)
                .filter(region -> region.contains(position))
                .collect(Collectors.toList());
    }

    /**
     * Gets the region with the highest priority among all involved regions at the given location and dimension. <br>
     * This considers the active state of the region as well. <br>
     *
     * @param position the position to check for involved regions
     * @param dim      the dimension to check for involved regions
     * @return the region with the highest priority among all involved regions which contain the given location
     */
    @Nullable
    private static IMarkableRegion getInvolvedRegionFor(class_2338 position, class_5321<class_1937> dim) {
        List<IMarkableRegion> regionsForPos = getInvolvedRegionsFor(position, dim);
        if (regionsForPos.isEmpty()) {
            return null;
        } else {
            return Collections.max(regionsForPos, Comparator.comparing(IMarkableRegion::getPriority));
        }
    }

    /**
     * Gets the flag state for the given region and flag. <br>
     * If the player is null, the flag state is returned as is. <br>
     * If the player is not null, the flag state is checked against the player's permissions. <br>
     * If the player is permitted, the flag state is returned as allowed. <br>
     *
     * @param region the region to get the flag state for
     * @param flag   the flag to get the state for
     * @param player the player to check the flag state against
     * @return the flag state for the given region and flag
     */
    private static FlagState getFlagState(IProtectedRegion region, RegionFlag flag, @Nullable class_1657 player) {
        if (player == null) {
            return region.getFlagContainer().flagState(flag.name);
        } else {
            boolean hasPermission = Permissions.get().hasAnyPermission(region, player, Permissions.getGroups(region, player));
            boolean isPermitted = hasPermission || Permissions.get().hasConfigPermAndOpBypassFlags(player);
            if (isPermitted) {
                return FlagState.ALLOWED;
            } else {
                return region.getFlagContainer().flagState(flag.name);
            }
        }
    }

    public static Map<String, FlagMessage.FlagCorrelation> getFlagMapRecursive(IProtectedRegion region, Map<String, FlagMessage.FlagCorrelation> carry) {
        if (carry == null) {
            carry = region.getFlagContainer().entrySet().stream()
                    .filter(flag -> flag.getValue().getState() != FlagState.UNDEFINED)
                    .collect(Collectors.toMap(Map.Entry::getKey, entry -> new FlagMessage.FlagCorrelation(region, entry.getValue())));
        }
        if (region.equals(region.getParent())) {
            // global region has itself as parent
            Set<Map.Entry<String, IFlag>> flags = getNonUndefinedFlags(region);
            for (Map.Entry<String, IFlag> entry : flags) {
                if (!carry.containsKey(entry.getKey())) {
                    carry.put(entry.getValue().getName(), new FlagMessage.FlagCorrelation(region, entry.getValue()));
                }
            }
            return carry;
        }
        Set<Map.Entry<String, IFlag>> parentFlags = getNonUndefinedFlags(region.getParent());
        for (Map.Entry<String, IFlag> entry : parentFlags) {
            if (!carry.containsKey(entry.getKey())) {
                carry.put(entry.getValue().getName(), new FlagMessage.FlagCorrelation(region.getParent(), entry.getValue()));
            }
            if (entry.getValue().doesOverride()) {
                carry.put(entry.getValue().getName(), new FlagMessage.FlagCorrelation(region.getParent(), entry.getValue()));
            }
        }
        return getFlagMapRecursive(region.getParent(), carry);
    }

    private static Set<Map.Entry<String, IFlag>> getNonUndefinedFlags(IProtectedRegion region) {
        return region.getFlagContainer().entrySet().stream()
                .filter(flag -> flag.getValue().getState() != FlagState.UNDEFINED)
                .collect(Collectors.toSet());
    }


    /**
     * Recursively gets all flags defined in the region hierarchy of the provided region, including all it's parents. <br>
     * This is used, when we have determined the responsible region for an Event or a BlockPos. <br>
     * We need to check all flags of the responsible region and all its parents to determine the final flag state. <br>
     *
     * @param region the region (and its parents) to get active flags
     * @param carry  a flag container, holding information about region flags
     * @return a flag container of all active flags of the given region including its parents.
     */
    public static FlagContainer getFlagsRecursive(IProtectedRegion region, FlagContainer carry) {
        if (region.equals(region.getParent())) { // global region has itself as parent
            return carry == null ? region.getFlagContainer().deepCopy() : carry;
        }
        if (carry == null) { // effectively make a deep copy of the flag container
            carry = region.getFlagContainer().deepCopy();
        }
        Map<String, IFlag> activeParentFlags = region.getParent().getFlagContainer().getActiveFlags();
        for (Map.Entry<String, IFlag> entry : activeParentFlags.entrySet()) {
            String flagName = entry.getKey();
            IFlag flag = entry.getValue();
            boolean parentFlagOverrides = flag.doesOverride();
            boolean existingFlag = carry.contains(flagName);
            if (parentFlagOverrides && existingFlag) {
                carry.remove(flagName);
                carry.put(flagName, flag);
            }
            if (!existingFlag) {
                carry.put(flagName, flag);
            }
        }
        return getFlagsRecursive(region.getParent(), carry);
    }

    /**
     * Recursively gets the region hierarchy for the given region. <br>
     * The region hierarchy is a list of regions starting with the given region and ending with the global region. <br>
     *
     * @param region the region to get the hierarchy for
     * @param carry  the list to carry the hierarchy (initially empty)
     * @return the region hierarchy for the given region
     */
    public static List<IProtectedRegion> getRegionHierarchy(IProtectedRegion region, List<IProtectedRegion> carry) {
        if (region.equals(region.getParent())) { // global region has itself as parent
            return carry.isEmpty() ? Collections.singletonList(region) : carry;
        }
        carry.add(region);
        return getRegionHierarchy(region.getParent(), carry);
    }

    /**
     * Recursively gets the flag correlation for the given region and flag. <br>
     * The flag correlation is the region and flag pair which is responsible for the flag state. <br>
     * The flag correlation is determined by checking the flag state of the region and its parents. <br>
     * If the flag is allowed or denied, the flag correlation is set. <br>
     * If the flag is undefined, the parent region is checked. <br>
     * If the parent region is the global region, the flag correlation is returned. <br>
     *
     * @param region     the region to get the flag correlation for
     * @param regionFlag the region flag to get the correlation for
     * @param carry      the flag correlation to carry (initially null)
     * @return the flag correlation for the given region and flag
     */
    private static FlagMessage.FlagCorrelation getFlagCorrelation(IProtectedRegion region, RegionFlag regionFlag, @Nullable FlagMessage.FlagCorrelation carry) {
        if (region.equals(region.getParent())) {
            if (region.getFlagContainer().flagState(regionFlag.name) != FlagState.UNDEFINED) {
                IFlag flag = region.getFlag(regionFlag.name);
                if (flag.doesOverride()) {
                    carry = new FlagMessage.FlagCorrelation(region, flag);
                }
            }
            return carry;
        }
        FlagState flagState = region.getFlagContainer().flagState(regionFlag.name);
        if (flagState != FlagState.UNDEFINED) {
            // allowed or denied
            carry = new FlagMessage.FlagCorrelation(region, region.getFlag(regionFlag.name));
        }
        return getFlagCorrelation(region.getParent(), regionFlag, carry);
    }

    public static FlagMessage.FlagCorrelation getResponsibleFlag(IProtectedRegion region, RegionFlag regionFlag, @Nullable FlagMessage.FlagCorrelation carry) {
        if (carry == null) {
            FlagState flagState = region.getFlagContainer().flagState(regionFlag.name);
            if (flagState == FlagState.ALLOWED || flagState == FlagState.DENIED) {
                IFlag flag = region.getFlag(regionFlag.name);
                carry = new FlagMessage.FlagCorrelation(region, flag);
            } else
                carry = new FlagMessage.FlagCorrelation(region, null);
        }
        if (region.equals(region.getParent())) {
            if (region.getFlagContainer().flagState(regionFlag.name) != FlagState.UNDEFINED) {
                if (carry.getFlag() == null) {
                    carry = new FlagMessage.FlagCorrelation(region, region.getFlag(regionFlag.name));
                }
            }
            return carry;
        }
        FlagState flagState = region.getParent().getFlagContainer().flagState(regionFlag.name);
        if (flagState == FlagState.ALLOWED || flagState == FlagState.DENIED) {
            IFlag flag = region.getParent().getFlag(regionFlag.name);
            if (carry.getFlag() == null) {
                carry = new FlagMessage.FlagCorrelation(region.getParent(), flag);
            }
            if (flag.doesOverride()) {
                carry = new FlagMessage.FlagCorrelation(region.getParent(), flag);
            }
        }
        return getResponsibleFlag(region.getParent(), regionFlag, carry);
    }
}

==> yawp-1.21.5-fabric-0.5.3-beta1-sources.jar: de/z0rdak/yawp/handler/HandlerUtil.java <==
package de.z0rdak.yawp.handler;

import de.z0rdak.yawp.api.permission.Permissions;
import de.z0rdak.yawp.core.flag.*;
import de.z0rdak.yawp.core.region.IProtectedRegion;
import org.jetbrains.annotations.Nullable;

import java.util.*;
import java.util.stream.Collectors;
import net.minecraft.class_1297;
import net.minecraft.class_1307;
import net.minecraft.class_1429;
import net.minecraft.class_1480;
import net.minecraft.class_1506;
import net.minecraft.class_1507;
import net.minecraft.class_1510;
import net.minecraft.class_1569;
import net.minecraft.class_1606;
import net.minecraft.class_1621;
import net.minecraft.class_1657;
import net.minecraft.class_1915;
import net.minecraft.class_1936;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_5321;

public final class HandlerUtil {

    private HandlerUtil() {
    }

    public static class_5321<class_1937> getDimKey(class_1297 entity) {
        return entity.method_37908().method_27983();
    }

    public static class_5321<class_1937> getDimKey(class_1937 world) {
        return world.method_27983();
    }

    public static boolean isAnimal(class_1297 entity) {
        boolean isMonsterHorse = entity instanceof class_1507 || entity instanceof class_1506;
        // Note: because monster horses are still horses and horses are animals? what the frick mojang
        return entity instanceof class_1429 && !isMonsterHorse 
                || entity instanceof class_1480;
    }

    public static boolean isServerSide(class_1297 entity) {
        return isServerSide(entity.method_37908());
    }

    public static boolean isServerSide(class_1936 world) {
        return !world.method_8608();
    }
    public static boolean isServerSide(class_1937 world) {
        return !world.field_9236;
    }

    public static boolean isVillager(class_1297 entity) {
        return entity instanceof class_1915;
    }

    public static boolean isPlayer(class_1297 entity) {
        return entity instanceof class_1657;
    }

    public static boolean isMonster(class_1297 entity) {
        return entity instanceof class_1569
                || entity instanceof class_1621
                || entity instanceof class_1307
                || entity instanceof class_1510
                || entity instanceof class_1606 
                || entity instanceof class_1507 || entity instanceof class_1506;
    }

    public static boolean notServerSideOrPlayerNull(class_1297 entity) {
        return entity == null || !isServerSide(entity);
    }

    public static void syncPlayerInventory(class_1937 world, class_1657 player) {
        // TODO:
    }

    public static void updateBlockState(class_1937 world, class_2338 pos) {
        world.method_8408(pos, world.method_8320(pos).method_26204());
    }


    public static Map<String, FlagCorrelation> getFlagMapRecursive(IProtectedRegion region, Map<String, FlagCorrelation> carry) {
        if (carry == null) {
            carry = region.getFlags().flagEntries().stream()
                    .filter(flag -> flag.getValue().getState() != FlagState.UNDEFINED)
                    .collect(Collectors.toMap(Map.Entry::getKey, entry -> new FlagCorrelation(region, entry.getValue())));
        }
        if (region.equals(region.getParent())) {
            // global region has itself as parent
            Set<Map.Entry<String, IFlag>> flags = getNonUndefinedFlags(region);
            for (Map.Entry<String, IFlag> entry : flags) {
                if (!carry.containsKey(entry.getKey())) {
                    carry.put(entry.getValue().getName(), new FlagCorrelation(region, entry.getValue()));
                }
            }
            return carry;
        }
        Set<Map.Entry<String, IFlag>> parentFlags = getNonUndefinedFlags(region.getParent());
        for (Map.Entry<String, IFlag> entry : parentFlags) {
            if (!carry.containsKey(entry.getKey())) {
                carry.put(entry.getValue().getName(), new FlagCorrelation(region.getParent(), entry.getValue()));
            }
            if (entry.getValue().doesOverride()) {
                carry.put(entry.getValue().getName(), new FlagCorrelation(region.getParent(), entry.getValue()));
            }
        }
        return getFlagMapRecursive(region.getParent(), carry);
    }

    private static Set<Map.Entry<String, IFlag>> getNonUndefinedFlags(IProtectedRegion region) {
        return region.getFlags().flagEntries().stream()
                .filter(flag -> flag.getValue().getState() != FlagState.UNDEFINED)
                .collect(Collectors.toSet());
    }

}
